﻿using CaseAccountingContracts.BindingModels;
using CaseAccountingContracts.BusinessLogicContracts;
using CaseAccountingContracts.SearchModels;
using CaseAccountingContracts.StoragesContracts;
using CaseAccountingContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CaseAccountingBusinessLogic.BusinessLogics
{
    public class HearingLogic : IHearingLogic
    {
        private readonly ILogger _logger;
        private readonly IHearingStorage _hearingStorage;

        public HearingLogic(ILogger<HearingLogic> logger, IHearingStorage hearingStorage)
        {
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
            _hearingStorage = hearingStorage ?? throw new ArgumentNullException(nameof(hearingStorage));
        }

        public bool Create(HearingBindingModel model)
        {
            CheckModel(model);
            if (_hearingStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(HearingBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_hearingStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public HearingViewModel? ReadElement(HearingSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _hearingStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<HearingViewModel>? ReadList(HearingSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _hearingStorage.GetFullList() : _hearingStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(HearingBindingModel model)
        {
            CheckModel(model);
            if (_hearingStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(HearingBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор пользователя",
                    nameof(model.UserId));
            }
            if (model.CaseId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор дела",
                    nameof(model.CaseId));
            }
            if (model.Information == string.Empty) 
            {
                throw new ArgumentNullException("Некорректна написана информация по делу",
                    nameof(model.Information));
            }
        }
    }
}
